==> 0.0.21.tar.gz: android/src/main/java/com/antsomi/antsomi/AntsomiPlugin.java <==
package com.antsomi.antsomi;

import android.app.Activity;
import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.os.Parcelable;
import android.util.Log;
import android.os.Handler;
import android.os.Looper;

import androidx.annotation.NonNull;

import com.antsomi.APICallback;
import com.antsomi.AntsomiSdk;
import com.antsomi.AntsomiTrackEvent;
import com.antsomi.AppInbox.Catalog;
import com.antsomi.AppInbox.MessageInbox;
import com.antsomi.AppInbox.OnNewMessageCallback;
import static com.antsomi.Utils.convertHashMapToJson;
// import static com.antsomi.Utils.createMap;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;

import java.lang.ref.WeakReference;

/** AntsomiPlugin */
public class AntsomiPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
  private MethodChannel channel;
  private Activity activity;
  private Context context;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "antsomi");
    channel.setMethodCallHandler(this);

    this.context = flutterPluginBinding.getApplicationContext();
  }

  @Override
  public void onAttachedToActivity(ActivityPluginBinding binding) {
    this.context = binding.getActivity();
  }

  @Override
  public void onReattachedToActivityForConfigChanges(ActivityPluginBinding binding) {
    this.context = binding.getActivity();
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {

  }

  @Override
  public void onDetachedFromActivity() {

  }

  private static class MethodResultWrapper implements Result {
    private Result methodResult;
    private Handler handler;

    MethodResultWrapper(Result result) {
      methodResult = result;
      handler = new Handler(Looper.getMainLooper());
    }

    @Override
    public void success(final Object result) {
      handler.post(new Runnable() {
        @Override
        public void run() {
          methodResult.success(result);
        }
      });
    }

    @Override
    public void error(final String errorCode, final String errorMessage, final Object errorDetails) {
      handler.post(new Runnable() {
        @Override
        public void run() {
          methodResult.error(errorCode, errorMessage, errorDetails);
        }
      });
    }

    @Override
    public void notImplemented() {
      handler.post(new Runnable() {
        @Override
        public void run() {
          methodResult.notImplemented();
        }
      });
    }
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result rawResult) {

    Result result = new MethodResultWrapper(rawResult);

    switch (call.method) {
      case "getPlatformVersion":
        result.success("Android " + android.os.Build.VERSION.RELEASE);
        break;
      case "config":
        String portalId = call.argument("portalId");
        String propsId = call.argument("propsId");
        String appId = call.argument("applicationId");

        AntsomiSdk.Builder builder = new AntsomiSdk.Builder(this.context, portalId, propsId,
            appId);

        try {
          builder.build();

          AntsomiSdk.getInstance().setTrackingScreenView(false);
          AntsomiSdk.getInstance().setIsShowTemplate(false);
        } catch (Exception e) {
          throw new RuntimeException(e);
        }
        break;
      case "getUID":
        result.success(AntsomiSdk.getInstance().getUidPush());
        break;
      case "getCustomerId":
        result.success(AntsomiSdk.getInstance().getCustomerId());
        break;
      case "getPropsId":
        result.success(AntsomiSdk.getInstance().getPropsId());
        break;
      case "getPortalId":
        result.success(AntsomiSdk.getInstance().getPortalId());
        break;
      case "appInboxInit":
        String destinationId = call.argument("destinationId");
        String audienceType = call.argument("audienceType");
        AntsomiSdk.getInstanceAppInbox().init(destinationId, audienceType);
        break;
      case "getMessage":
        List<String> labels = call.argument("catalogId");
        int page = call.argument("page");

        AntsomiSdk.getInstanceAppInbox().getListNotification(labels, page, new APICallback<List<MessageInbox>>() {
          @Override
          public void onResponse(List<MessageInbox> response) {
            List<Map<String, String>> messages = new ArrayList<>();

            for (MessageInbox item : response) {
              try {
                messages.add(toJsonInboxItem(item));
              } catch (JSONException e) {
                List<Map<String, String>> messageEmpty = new ArrayList<>();
                result.success(messageEmpty);
              }
            }

            result.success(messages);
          }

          @Override
          public void onFailure(Throwable t) {
            List<Map<String, String>> messages = new ArrayList<>();
            result.success(messages);
          }
        });

        break;
      case "setPageLimit":
        int limit = call.argument("limit");
        AntsomiSdk.getInstanceAppInbox().setPageLimit(limit);
        break;
      case "getUnreadMessageCount":
        AntsomiSdk.getInstanceAppInbox().getCountNotificationUnread(new APICallback<Integer>() {
          @Override
          public void onResponse(Integer response) {
            result.success(response);
          }

          @Override
          public void onFailure(Throwable t) {
            // Handle network failures or other errors
            result.success(0);
          }
        });
        break;
      case "setCustomerProperties":
        String customerId = call.argument("customerId");
        AntsomiSdk.getInstance().setCustomerId(customerId);
        break;
      case "getMessageById":
        String messageId = call.argument("messageId");
        AntsomiSdk.getInstanceAppInbox().getDetailMessage(messageId, new APICallback<MessageInbox>() {
          @Override
          public void onResponse(MessageInbox response) {
            Map<String, String> jsonMessage = null;
            try {
              jsonMessage = toJsonInboxItem(response);
            } catch (JSONException e) {
              throw new RuntimeException(e);
            }
            result.success(jsonMessage);
          }

          @Override
          public void onFailure(Throwable t) {
            Map<String, String> message = new HashMap<>();

            result.success(message);
          }
        });
        break;
      case "getAllLabels":
        AntsomiSdk.getInstanceAppInbox().getAllLabels(new APICallback<List<Catalog>>() {
          @Override
          public void onResponse(List<Catalog> response) {
            List<String> catalogs = new ArrayList<>();

            for (Catalog item : response) {
              catalogs.add(toJsonStringCatalog(item));
            }

            result.success(catalogs);
          }

          @Override
          public void onFailure(Throwable t) {
            List<String> catalogs = new ArrayList<>();
            result.success(catalogs);
          }
        });
        break;
      case "modifyAction":
        ArrayList<String> messageIds = call.argument("messageId");
        int action = call.argument("action");

        Log.d("messageIds", messageIds.toString());
        Log.d("action", String.valueOf(action));

        if(action == MessageInbox.MessageStatus.READ.getValue()){
          AntsomiSdk.getInstanceAppInbox().updateMessage(messageIds, MessageInbox.MessageStatus.READ);
        } else if (action == MessageInbox.MessageStatus.DELETE.getValue()){
          AntsomiSdk.getInstanceAppInbox().updateMessage(messageIds, MessageInbox.MessageStatus.DELETE);
        }

        result.success(true);


        break;
      case "track":
        Map<String, Object> event = call.argument("event");

        if (event != null) {
          Object eventName = event.get("en");
          if (eventName == null) {
            Log.w("EventName", "Event name cannot be null");
            return;
          }
          String eventNameEvent = eventName.toString();

          AntsomiTrackEvent atEvent = new AntsomiTrackEvent(eventNameEvent);

          Object eventProps = event.get("eventProps");
          Object customerProps = event.get("customerProps");
          Object objectProps = event.get("objectProps");
          Object userProps = event.get("userProps");
          Object items = event.get("items");


          if (eventProps instanceof HashMap) {
            Map<String, Object> eventPropsMap = (HashMap) eventProps;

            atEvent.setEventProperties(convertMapToBundle(eventPropsMap));
          }

          if (customerProps instanceof HashMap) {
            Map<String, Object> customerPropsMap = (HashMap) customerProps;

            atEvent.setCustomerProperties(convertMapToBundle(customerPropsMap));
          }

          if (userProps instanceof HashMap) {

            Map<String, Object> userPropsMap = (HashMap) userProps;

            atEvent.setUserProperties(convertMapToBundle(userPropsMap));

          }

          if (objectProps instanceof HashMap) {
            Map<String, Object> objectPropsMap = (HashMap) objectProps;

            atEvent.setObjectProperties(convertMapToBundle(objectPropsMap));
          }

          ArrayList<Bundle> itemsEvent = new ArrayList<Bundle>();
          if (items != null) {
            if (items instanceof ArrayList) {
              ArrayList<?> arrayList = (ArrayList<?>) items;

              if (!arrayList.isEmpty() && arrayList.get(0) instanceof Map) {
                ArrayList<Map<String, Object>> castedList = (ArrayList<Map<String, Object>>) arrayList;

                for (Map<String, Object> item : castedList) {
                  Bundle bundle = new Bundle();
                  for (Map.Entry<String, Object> entry : item.entrySet()) {
                    Object value = entry.getValue();
                    if (value instanceof String) {
                      bundle.putString(entry.getKey(), (String) value);
                    } else if (value instanceof Integer) {
                      bundle.putInt(entry.getKey(), (Integer) value);
                    } else if (value instanceof Double) {
                      bundle.putDouble(entry.getKey(), (Double) value);
                    } else if (value instanceof Float) {
                      bundle.putFloat(entry.getKey(), (Float) value);
                    }
                  }

                  itemsEvent.add(bundle);
                }

                atEvent.setItems(itemsEvent);
              }
            }
          }

          AntsomiSdk.getInstance().track(atEvent);
        }
        break;
      case "trackScreen":
        String screenName = call.argument("name");
        String screenTitle = call.argument("title");
        String screenType = call.argument("type");

        AntsomiSdk.getInstance().trackingScreen(screenName, screenTitle, screenType);
        break;
      case "parseDeepLink":
        String url = call.argument("url");
        AntsomiSdk.getInstance().handleDeeplinkURL(url);
        break;
      case "getDeviceId":
        String deviceId = AntsomiSdk.getInstance().getDeviceStringId();
        result.success(deviceId);
        break;
      case "newMessageReceived":
        onNewMessageReceive();
        break;
      case "handleTrackingUrl":
        String trackingUrl = call.argument("trackingUrl");
        AntsomiSdk.handleTrackingUrl(trackingUrl);
        break;

      default:
        result.notImplemented();
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }

  private void onNewMessageReceive() {
    AntsomiSdk.getInstanceAppInbox().addOnNewMessageCallback(new OnNewMessageCallback() {
      @Override
      public void onNewMessage(MessageInbox message) {
        runOnMainUIThread(new Runnable() {
          @Override
          public void run() {
            try {
              channel.invokeMethod("onReceiveNewData", toJsonInboxItem(message));
            } catch (JSONException e) {
              throw new RuntimeException(e);
            }
          }
        });

      }

      @Override
      public void run() {

      }
    });
  }

  private void runOnMainUIThread(Runnable runnable) {
    if (Looper.getMainLooper().getThread() == Thread.currentThread())
      runnable.run();
    else {
      Handler handler = new Handler(Looper.getMainLooper());
      handler.post(runnable);
    }
  }

  private Map<String, String> toJsonInboxItem(MessageInbox inboxItem) throws JSONException {
    Map<String, String> jsonEncode = new HashMap<>();
    jsonEncode.put("button_app_url_2", inboxItem.getButtonAppUrl2());
    jsonEncode.put("button_app_url_1", inboxItem.getButtonAppUrl1());
    jsonEncode.put("content", inboxItem.getContent());
    jsonEncode.put("last_updated", inboxItem.getLastUpdated());
    jsonEncode.put("app_url", inboxItem.getAppUrl());
    jsonEncode.put("item_id", inboxItem.getItemId());
    jsonEncode.put("heading", inboxItem.getHeading());
    jsonEncode.put("date_created", inboxItem.getDateCreated());
    jsonEncode.put("image_url", inboxItem.getImageUrl());
    jsonEncode.put("button_label_1", inboxItem.getButtonLabel1());

    if (inboxItem.getCatalogIds() != null) {
      jsonEncode.put("catalog_ids", new JSONArray(inboxItem.getCatalogIds()).toString());
    } else {
      jsonEncode.put("catalog_ids", new JSONArray().toString());
    }
    jsonEncode.put("button_label_2", inboxItem.getButtonLabel2());
    jsonEncode.put("launch_url", inboxItem.getLaunchUrl());
    jsonEncode.put("button_launch_url_1", inboxItem.getButtonLaunchUrl1());
    jsonEncode.put("button_launch_url_2", inboxItem.getButtonLaunchUrl2());
    jsonEncode.put("lookup_type", inboxItem.getLookupType());
    jsonEncode.put("template_id", inboxItem.getTemplateId());
    jsonEncode.put("status", inboxItem.getStatus());
    jsonEncode.put("tracking_url", inboxItem.getTrackingUrl());

    return jsonEncode;
  }

  private String toJsonStringCatalog(Catalog catalog) {
    HashMap<String, Object> jsonEncode = new HashMap<>();
    jsonEncode.put("catalogId", catalog.getCatalogId());
    jsonEncode.put("catalogName", catalog.getCatalogName());

    return convertHashMapToJson(jsonEncode);
  }

  private Bundle convertMapToBundle(Map<String, Object> dataMap) {
    Bundle bundle = new Bundle();

    for (Map.Entry<String, Object> entry : dataMap.entrySet()) {
      String key = entry.getKey();
      Object value = entry.getValue();

      if (value instanceof String) {
        bundle.putString(key, (String) value);
      } else if (value instanceof Integer) {
        bundle.putInt(key, (Integer) value);
      } else if (value instanceof Map) {
        Bundle nestedBundle = convertMapToBundle((Map<String, Object>) value);
        bundle.putBundle(key, nestedBundle);
      }
    }

    return bundle;
  }
}

==> 0.0.23.tar.gz: android/src/main/java/com/antsomi/antsomi/AntsomiPlugin.java <==
package com.antsomi.antsomi;

import android.app.Activity;
import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.os.Parcelable;
import android.util.Log;
import android.os.Handler;
import android.os.Looper;

import androidx.annotation.NonNull;

import com.antsomi.APICallback;
import com.antsomi.AntsomiSdk;
import com.antsomi.AntsomiTrackEvent;
import com.antsomi.AppInbox.Catalog;
import com.antsomi.AppInbox.MessageInbox;
import com.antsomi.AppInbox.OnNewMessageCallback;
import static com.antsomi.Utils.convertHashMapToJson;
// import static com.antsomi.Utils.createMap;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;

import java.lang.ref.WeakReference;

/** AntsomiPlugin */
public class AntsomiPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
  private MethodChannel channel;
  private Activity activity;
  private Context context;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "antsomi");
    channel.setMethodCallHandler(this);

    this.context = flutterPluginBinding.getApplicationContext();
  }

  @Override
  public void onAttachedToActivity(ActivityPluginBinding binding) {
    this.context = binding.getActivity();
  }

  @Override
  public void onReattachedToActivityForConfigChanges(ActivityPluginBinding binding) {
    this.context = binding.getActivity();
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {

  }

  @Override
  public void onDetachedFromActivity() {

  }

  private static class MethodResultWrapper implements Result {
    private Result methodResult;
    private Handler handler;

    MethodResultWrapper(Result result) {
      methodResult = result;
      handler = new Handler(Looper.getMainLooper());
    }

    @Override
    public void success(final Object result) {
      handler.post(new Runnable() {
        @Override
        public void run() {
          methodResult.success(result);
        }
      });
    }

    @Override
    public void error(final String errorCode, final String errorMessage, final Object errorDetails) {
      handler.post(new Runnable() {
        @Override
        public void run() {
          methodResult.error(errorCode, errorMessage, errorDetails);
        }
      });
    }

    @Override
    public void notImplemented() {
      handler.post(new Runnable() {
        @Override
        public void run() {
          methodResult.notImplemented();
        }
      });
    }
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result rawResult) {

    Result result = new MethodResultWrapper(rawResult);

    switch (call.method) {
      case "getPlatformVersion":
        result.success("Android " + android.os.Build.VERSION.RELEASE);
        break;
      case "config":
        String portalId = call.argument("portalId");
        String propsId = call.argument("propsId");
        String appId = call.argument("applicationId");

        AntsomiSdk.Builder builder = new AntsomiSdk.Builder(this.context, portalId, propsId,
            appId);

        try {
          builder.build();

          AntsomiSdk.getInstance().setTrackingScreenView(false);
        } catch (Exception e) {
          throw new RuntimeException(e);
        }
        break;
      case "getUID":
        result.success(AntsomiSdk.getInstance().getUidPush());
        break;
      case "getCustomerId":
        result.success(AntsomiSdk.getInstance().getCustomerId());
        break;
      case "getPropsId":
        result.success(AntsomiSdk.getInstance().getPropsId());
        break;
      case "getPortalId":
        result.success(AntsomiSdk.getInstance().getPortalId());
        break;
      case "appInboxInit":
        String destinationId = call.argument("destinationId");
        String audienceType = call.argument("audienceType");
        AntsomiSdk.getInstanceAppInbox().init(destinationId, audienceType);
        break;
      case "getMessage":
        List<String> labels = call.argument("catalogId");
        int page = call.argument("page");

        AntsomiSdk.getInstanceAppInbox().getListNotification(labels, page, new APICallback<List<MessageInbox>>() {
          @Override
          public void onResponse(List<MessageInbox> response) {
            List<Map<String, String>> messages = new ArrayList<>();

            for (MessageInbox item : response) {
              try {
                messages.add(toJsonInboxItem(item));
              } catch (JSONException e) {
                List<Map<String, String>> messageEmpty = new ArrayList<>();
                result.success(messageEmpty);
              }
            }

            result.success(messages);
          }

          @Override
          public void onFailure(Throwable t) {
            List<Map<String, String>> messages = new ArrayList<>();
            result.success(messages);
          }
        });

        break;
      case "setPageLimit":
        int limit = call.argument("limit");
        AntsomiSdk.getInstanceAppInbox().setPageLimit(limit);
        break;
      case "getUnreadMessageCount":
        AntsomiSdk.getInstanceAppInbox().getCountNotificationUnread(new APICallback<Integer>() {
          @Override
          public void onResponse(Integer response) {
            result.success(response);
          }

          @Override
          public void onFailure(Throwable t) {
            // Handle network failures or other errors
            result.success(0);
          }
        });
        break;
      case "setCustomerProperties":
        String customerId = call.argument("customerId");
        AntsomiSdk.getInstance().setCustomerId(customerId);
        break;
      case "getMessageById":
        String messageId = call.argument("messageId");
        AntsomiSdk.getInstanceAppInbox().getDetailMessage(messageId, new APICallback<MessageInbox>() {
          @Override
          public void onResponse(MessageInbox response) {
            Map<String, String> jsonMessage = null;
            try {
              jsonMessage = toJsonInboxItem(response);
            } catch (JSONException e) {
              throw new RuntimeException(e);
            }
            result.success(jsonMessage);
          }

          @Override
          public void onFailure(Throwable t) {
            Map<String, String> message = new HashMap<>();

            result.success(message);
          }
        });
        break;
      case "setIsDelivery":
        boolean isDelivery = Boolean.TRUE.equals(call.argument("value"));
        AntsomiSdk.getInstance().setIsShowTemplate(isDelivery);
        break;
      case "getAllLabels":
        AntsomiSdk.getInstanceAppInbox().getAllLabels(new APICallback<List<Catalog>>() {
          @Override
          public void onResponse(List<Catalog> response) {
            List<String> catalogs = new ArrayList<>();

            for (Catalog item : response) {
              catalogs.add(toJsonStringCatalog(item));
            }

            result.success(catalogs);
          }

          @Override
          public void onFailure(Throwable t) {
            List<String> catalogs = new ArrayList<>();
            result.success(catalogs);
          }
        });
        break;
      case "modifyAction":
        ArrayList<String> messageIds = call.argument("messageId");
        int action = call.argument("action");

        if(action == MessageInbox.MessageStatus.READ.getValue()){
          AntsomiSdk.getInstanceAppInbox().updateMessage(messageIds, MessageInbox.MessageStatus.READ);
        } else if (action == MessageInbox.MessageStatus.DELETE.getValue()){
          AntsomiSdk.getInstanceAppInbox().updateMessage(messageIds, MessageInbox.MessageStatus.DELETE);
        }

        result.success(true);


        break;
      case "track":
        Map<String, Object> event = call.argument("event");

        if (event != null) {
          Object eventName = event.get("en");
          if (eventName == null) {
            Log.w("EventName", "Event name cannot be null");
            return;
          }
          String eventNameEvent = eventName.toString();

          AntsomiTrackEvent atEvent = new AntsomiTrackEvent(eventNameEvent);

          Object eventProps = event.get("eventProps");
          Object customerProps = event.get("customerProps");
          Object objectProps = event.get("objectProps");
          Object userProps = event.get("userProps");
          Object items = event.get("items");


          if (eventProps instanceof HashMap) {
            Map<String, Object> eventPropsMap = (HashMap) eventProps;

            atEvent.setEventProperties(convertMapToBundle(eventPropsMap));
          }

          if (customerProps instanceof HashMap) {
            Map<String, Object> customerPropsMap = (HashMap) customerProps;

            atEvent.setCustomerProperties(convertMapToBundle(customerPropsMap));
          }

          if (userProps instanceof HashMap) {

            Map<String, Object> userPropsMap = (HashMap) userProps;

            atEvent.setUserProperties(convertMapToBundle(userPropsMap));

          }

          if (objectProps instanceof HashMap) {
            Map<String, Object> objectPropsMap = (HashMap) objectProps;

            atEvent.setObjectProperties(convertMapToBundle(objectPropsMap));
          }

          ArrayList<Bundle> itemsEvent = new ArrayList<Bundle>();
          if (items != null) {
            if (items instanceof ArrayList) {
              ArrayList<?> arrayList = (ArrayList<?>) items;

              if (!arrayList.isEmpty() && arrayList.get(0) instanceof Map) {
                ArrayList<Map<String, Object>> castedList = (ArrayList<Map<String, Object>>) arrayList;

                for (Map<String, Object> item : castedList) {
                  Bundle bundle = new Bundle();
                  for (Map.Entry<String, Object> entry : item.entrySet()) {
                    Object value = entry.getValue();
                    if (value instanceof String) {
                      bundle.putString(entry.getKey(), (String) value);
                    } else if (value instanceof Integer) {
                      bundle.putInt(entry.getKey(), (Integer) value);
                    } else if (value instanceof Double) {
                      bundle.putDouble(entry.getKey(), (Double) value);
                    } else if (value instanceof Float) {
                      bundle.putFloat(entry.getKey(), (Float) value);
                    }
                  }

                  itemsEvent.add(bundle);
                }

                atEvent.setItems(itemsEvent);
              }
            }
          }

          AntsomiSdk.getInstance().track(atEvent);
        }
        break;
      case "trackScreen":
        String screenName = call.argument("name");
        String screenTitle = call.argument("title");
        String screenType = call.argument("type");

        AntsomiSdk.getInstance().trackingScreen(screenName, screenTitle, screenType);
        break;
      case "parseDeepLink":
        String url = call.argument("url");
        AntsomiSdk.getInstance().handleDeeplinkURL(url);
        break;
      case "getDeviceId":
        String deviceId = AntsomiSdk.getInstance().getDeviceStringId();
        result.success(deviceId);
        break;
      case "newMessageReceived":
        onNewMessageReceive();
        break;
      case "handleTrackingUrl":
        String trackingUrl = call.argument("trackingUrl");
        AntsomiSdk.handleTrackingUrl(trackingUrl);
        break;

      default:
        result.notImplemented();
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }

  private void onNewMessageReceive() {
    AntsomiSdk.getInstanceAppInbox().addOnNewMessageCallback(new OnNewMessageCallback() {
      @Override
      public void onNewMessage(MessageInbox message) {
        runOnMainUIThread(new Runnable() {
          @Override
          public void run() {
            try {
              channel.invokeMethod("onReceiveNewData", toJsonInboxItem(message));
            } catch (JSONException e) {
              throw new RuntimeException(e);
            }
          }
        });

      }

      @Override
      public void run() {

      }
    });
  }

  private void runOnMainUIThread(Runnable runnable) {
    if (Looper.getMainLooper().getThread() == Thread.currentThread())
      runnable.run();
    else {
      Handler handler = new Handler(Looper.getMainLooper());
      handler.post(runnable);
    }
  }

  private Map<String, String> toJsonInboxItem(MessageInbox inboxItem) throws JSONException {
    Map<String, String> jsonEncode = new HashMap<>();
    jsonEncode.put("button_app_url_2", inboxItem.getButtonAppUrl2());
    jsonEncode.put("button_app_url_1", inboxItem.getButtonAppUrl1());
    jsonEncode.put("content", inboxItem.getContent());
    jsonEncode.put("last_updated", inboxItem.getLastUpdated());
    jsonEncode.put("app_url", inboxItem.getAppUrl());
    jsonEncode.put("item_id", inboxItem.getItemId());
    jsonEncode.put("heading", inboxItem.getHeading());
    jsonEncode.put("date_created", inboxItem.getDateCreated());
    jsonEncode.put("image_url", inboxItem.getImageUrl());
    jsonEncode.put("button_label_1", inboxItem.getButtonLabel1());

    if (inboxItem.getCatalogIds() != null) {
      jsonEncode.put("catalog_ids", new JSONArray(inboxItem.getCatalogIds()).toString());
    } else {
      jsonEncode.put("catalog_ids", new JSONArray().toString());
    }
    jsonEncode.put("button_label_2", inboxItem.getButtonLabel2());
    jsonEncode.put("launch_url", inboxItem.getLaunchUrl());
    jsonEncode.put("button_launch_url_1", inboxItem.getButtonLaunchUrl1());
    jsonEncode.put("button_launch_url_2", inboxItem.getButtonLaunchUrl2());
    jsonEncode.put("lookup_type", inboxItem.getLookupType());
    jsonEncode.put("template_id", inboxItem.getTemplateId());
    jsonEncode.put("status", inboxItem.getStatus());
    jsonEncode.put("tracking_url", inboxItem.getTrackingUrl());

    return jsonEncode;
  }

  private String toJsonStringCatalog(Catalog catalog) {
    HashMap<String, Object> jsonEncode = new HashMap<>();
    jsonEncode.put("catalogId", catalog.getCatalogId());
    jsonEncode.put("catalogName", catalog.getCatalogName());

    return convertHashMapToJson(jsonEncode);
  }

  private Bundle convertMapToBundle(Map<String, Object> dataMap) {
    Bundle bundle = new Bundle();

    for (Map.Entry<String, Object> entry : dataMap.entrySet()) {
      String key = entry.getKey();
      Object value = entry.getValue();

      if (value instanceof String) {
        bundle.putString(key, (String) value);
      } else if (value instanceof Integer) {
        bundle.putInt(key, (Integer) value);
      } else if (value instanceof Map) {
        Bundle nestedBundle = convertMapToBundle((Map<String, Object>) value);
        bundle.putBundle(key, nestedBundle);
      }
    }

    return bundle;
  }
}
